from __future__ import with_statement
import os
import re
import tempfile
import shutil

from fabric.api import run, local, get, cd, lcd, settings, hide
from fabric.state import env
from fabric.operations import require

#env.local_base_dir = os.path.dirname(os.path.abspath(__file__))

def get_wordpress_settings(settings_file):
    lines = open(os.path.join(env.local_base_dir, 'php', 'public_html', settings_file)).read()

    define_re = re.compile(r"""define\(["'](?P<key>[^'"]+)["'],\s*["'](?P<value>[^'"]+)["']\)""", re.S | re.M)

    return dict(define_re.findall(lines))


def update_from_remote():
    require('base_dir', provided_by=("env_test", "env_live"),
            used_for='defining the deploy environment')

    wp_settings = get_wordpress_settings(env.settings_file)
    local_wp_settings = get_wordpress_settings(os.path.join(env.local_base_dir, 'php', 'public_html', 'wp-config.php'))

    with lcd(os.path.join(env.local_base_dir, 'php', 'public_html')):
        local('rsync -a %s@%s:%s/public_html/wp-content/uploads/ wp-content/uploads/' % (env.user, env.host, env.base_dir))

    run('mysqldump -u%(DB_USER)s -p%(DB_PASSWORD)s -h%(DB_HOST)s --add-drop-table %(DB_NAME)s > /tmp/%(DB_USER)s.sql' % wp_settings)
    get('/tmp/%(DB_USER)s.sql' % wp_settings, '/tmp/%(path)s')
    run('rm -f /tmp/%(DB_USER)s.sql' % wp_settings)
    local_wp_settings['remote_file'] = '/tmp/%(DB_USER)s' % wp_settings

    local('mysql -u%(DB_USER)s -p%(DB_PASSWORD)s -h%(DB_HOST)s %(DB_NAME)s < %(remote_file)s.sql' % local_wp_settings)
    local('rm -f /tmp/%(DB_USER)s.sql' % wp_settings)


def deploy():
    require('base_dir', provided_by=("env_test", "env_live"),
            used_for='defining the deploy environment')

    deploy_archive_dir = tempfile.mkdtemp()

    with lcd(env.local_base_dir):
        local('git archive master php | tar -x -f- -C %s' % (deploy_archive_dir))

    local('rsync -a --exclude wp-config.php %s/php/* %s@%s:%s/' % (deploy_archive_dir, env.user, env.host, env.base_dir))

    if env.settings_file:
        with cd(os.path.join(env.base_dir, 'public_html')):
            run('cp %s wp-config.php' % env.settings_file)

            with settings(warn_only=True):
                with hide('stderr', 'stdout'):
                    run('chmod -R 777 wp-content/uploads/')
                    run('chmod -R 777 wp-content/themes/')

    shutil.rmtree(deploy_archive_dir)



#coding: utf-8
import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "fabric-coat",
    version = "0.1",
    author = "Rasmus Schlünsen",
    author_email = "rs@konstellation.dk",
    description = ("Deployment helpers for fabric"),
    license = "BSD",
    keywords = "fabric coat deployment rsync helper",
    url = "https://bitbucket.org/madssj/fabric-coat",
    packages=['coat', ],
    long_description=read('README'),
    package_dir = {'': 'src'},
    install_requires = ['fabric >= 1.0'],
)



from __future__ import with_statement

import os
import tempfile
import shutil

from fabric.api import run, local, cd, settings, lcd, prefix, hide
from fabric.operations import require
from fabric.state import env


def copy_revision(current_revision, revision):
    # test remote for the revision and skip re-copying already
    # existing revisions
    with settings(hide("stdout", "stderr", "warnings"), warn_only=True):
        exists = run("test -d %s/%s && echo 1" % (env.versions_dir, revision))

    if exists == "1":
        return

    # create a local temp directory and export a version of revision there
    # and move files around
    archive_dir = tempfile.mkdtemp()
    with lcd(env.local_base_dir):
        local("git archive %s django static | tar -x -C %s" % (revision, archive_dir))

    if env.settings_file:
        with lcd("%s/django/%s" % (archive_dir, env.django_appname)):
            local("cp %s localsettings.py" % env.settings_file)

    # copy all of local onto remote using rsync, use hard links to save
    # space for unmodified files
    rsync_cmd = "%s/* %s@%s:%s/%s/" % (archive_dir, env.user, env.host, env.versions_dir, revision)
    if current_revision:
        with cd(env.versions_dir):
            run("rsync -a --link-dest=../%(cur)s/ %(cur)s/ %(new)s" %
                {'cur': current_revision, 'new': revision})

        local("rsync -a -c --delete %s" % (rsync_cmd))
    else:
        local("rsync -a %s" % rsync_cmd)

    shutil.rmtree(archive_dir)


def bootstrap():
    require("base_dir", "versions_dir", provided_by=("env_test", "env_live"),
            used_for="defining the deploy environment")

    run("test -d %(base_dir)s || mkdir -p %(base_dir)s" % {"base_dir": env.base_dir})
    run("test -d %(base_dir)s/env || virtualenv --no-site-packages %(base_dir)s/env" % {"base_dir": env.base_dir})
    run("test -d %(versions_dir)s || mkdir -p %(versions_dir)s" % {"versions_dir": env.versions_dir})


def activate_revision(revision):
    require("base_dir", "versions_dir", provided_by=("env_test", "env_live"),
            used_for="defining the deploy environment")

    with cd("%s/%s" % (env.versions_dir, revision)):
        with prefix("source %s/env/bin/activate" % env.base_dir):
            if env.update_env:
                run("pip -q install -r django/requirements.txt")

            if env.syncdb:
                run("python django/%s/manage.py syncdb --noinput" % env.django_appname)

            if env.migrate:
                run("python django/%s/manage.py migrate" % env.django_appname)

    with cd(env.versions_dir):
        with settings(hide("warnings", "stderr"), warn_only=True):
            run("test -L current && rm current")

        run("ln -s %s current" % revision)

    with cd(env.base_dir):
        if env.wsgi_file:
            run("touch %s" % env.wsgi_file)


def version_resolve_current():
    revision = None
    with cd(env.versions_dir):
        with settings(hide("stderr", "running", "stdout", "warnings"), warn_only=True):
            revision = run("readlink current")
    return revision.rstrip("/") or None


def deploy(revision="HEAD"):
    require("base_dir", "versions_dir", provided_by=("env_test", "env_live"),
            used_for="defining the deploy environment")

    # resolve the incoming treeish hashref to an actual git revlog
    with hide("running"):
        revision = local("git log -1 --format=%%h %s" % revision, capture=True)

    # resolve the last delpoyed revision - will be None if first deployment
    current_revision = version_resolve_current()

    # create an archive of the revision, extract it on the remote
    # server, and rsync the difference - will be skipped if revision
    # already exists on remote
    copy_revision(current_revision, revision)

    # activate the new version - this might break migrations as
    # auto detecting the difference would be hard
    activate_revision(revision)
